@@ -3,7 +3,7 @@
   Copyright (C) 2019, 2021 Anastasis SARL
 
   Anastasis is free software; you can redistribute it and/or modify it under the
-  terms of the GNU Lesser General Public License as published by the Free Software
+  terms of the GNU Affero General Public License as published by the Free Software
   Foundation; either version 3, or (at your option) any later version.
 
   Anastasis is distributed in the hope that it will be useful, but WITHOUT ANY
@@ -219,7 +219,7 @@ sms_validate (void *cls,
   {
     if (MHD_NO ==
         TALER_MHD_reply_with_error (connection,
-                                    MHD_HTTP_EXPECTATION_FAILED,
+                                    MHD_HTTP_CONFLICT,
                                     TALER_EC_ANASTASIS_SMS_PHONE_INVALID,
                                     NULL))
       return GNUNET_SYSERR;
@@ -309,20 +309,17 @@ sms_done_cb (void *cls,
  * I.e. start to send SMS or e-mail or launch video identification.
  *
  * @param as authorization state
- * @param timeout how long do we have to produce a reply
  * @param connection HTTP client request (for queuing response, such as redirection to video portal)
  * @return state of the request
  */
-static enum ANASTASIS_AUTHORIZATION_Result
-sms_process (struct ANASTASIS_AUTHORIZATION_State *as,
-             struct GNUNET_TIME_Absolute timeout,
-             struct MHD_Connection *connection)
+static enum ANASTASIS_AUTHORIZATION_ChallengeResult
+sms_challenge (struct ANASTASIS_AUTHORIZATION_State *as,
+               struct MHD_Connection *connection)
 {
   MHD_RESULT mres;
   const char *mime;
   const char *lang;
 
-  (void) timeout;
   mime = MHD_lookup_connection_value (connection,
                                       MHD_HEADER_KIND,
                                       MHD_HTTP_HEADER_ACCEPT);
@@ -347,8 +344,8 @@ sms_process (struct ANASTASIS_AUTHORIZATION_State *as,
                                          TALER_EC_ANASTASIS_SMS_HELPER_EXEC_FAILED,
                                          "pipe");
       if (MHD_YES != mres)
-        return ANASTASIS_AUTHORIZATION_RES_FAILED_REPLY_FAILED;
-      return ANASTASIS_AUTHORIZATION_RES_FAILED;
+        return ANASTASIS_AUTHORIZATION_CRES_FAILED_REPLY_FAILED;
+      return ANASTASIS_AUTHORIZATION_CRES_FAILED;
     }
     as->child = GNUNET_OS_start_process (GNUNET_OS_INHERIT_STD_ERR,
                                          p,
@@ -366,26 +363,17 @@ sms_process (struct ANASTASIS_AUTHORIZATION_State *as,
                                          TALER_EC_ANASTASIS_SMS_HELPER_EXEC_FAILED,
                                          "exec");
       if (MHD_YES != mres)
-        return ANASTASIS_AUTHORIZATION_RES_FAILED_REPLY_FAILED;
-      return ANASTASIS_AUTHORIZATION_RES_FAILED;
+        return ANASTASIS_AUTHORIZATION_CRES_FAILED_REPLY_FAILED;
+      return ANASTASIS_AUTHORIZATION_CRES_FAILED;
     }
     pipe_stdin = GNUNET_DISK_pipe_detach_end (p,
                                               GNUNET_DISK_PIPE_END_WRITE);
     GNUNET_assert (NULL != pipe_stdin);
     GNUNET_DISK_pipe_close (p);
-    {
-      char *tpk;
-
-      tpk = GNUNET_STRINGS_data_to_string_alloc (
-        &as->truth_uuid,
-        sizeof (as->truth_uuid));
-      GNUNET_asprintf (&as->msg,
-                       "A-%llu\nAnastasis\n%s",
-                       (unsigned long long) as->code,
-                       tpk);
-      GNUNET_free (tpk);
-    }
-
+    GNUNET_asprintf (&as->msg,
+                     "%s\nAnastasis\n: %s",
+                     ANASTASIS_pin2s (as->code),
+                     ANASTASIS_CRYPTO_uuid2s (&as->truth_uuid));
     {
       const char *off = as->msg;
       size_t left = strlen (off);
@@ -406,8 +394,8 @@ sms_process (struct ANASTASIS_AUTHORIZATION_State *as,
                                              TALER_EC_ANASTASIS_SMS_HELPER_EXEC_FAILED,
                                              "write");
           if (MHD_YES != mres)
-            return ANASTASIS_AUTHORIZATION_RES_FAILED_REPLY_FAILED;
-          return ANASTASIS_AUTHORIZATION_RES_FAILED;
+            return ANASTASIS_AUTHORIZATION_CRES_FAILED_REPLY_FAILED;
+          return ANASTASIS_AUTHORIZATION_CRES_FAILED;
         }
         as->msg_off += ret;
         off += ret;
@@ -420,14 +408,14 @@ sms_process (struct ANASTASIS_AUTHORIZATION_State *as,
                                  as);
     as->connection = connection;
     MHD_suspend_connection (connection);
-    return ANASTASIS_AUTHORIZATION_RES_SUSPENDED;
+    return ANASTASIS_AUTHORIZATION_CRES_SUSPENDED;
   }
   if (NULL != as->cwh)
   {
     /* Spurious call, why are we here? */
     GNUNET_break (0);
     MHD_suspend_connection (connection);
-    return ANASTASIS_AUTHORIZATION_RES_SUSPENDED;
+    return ANASTASIS_AUTHORIZATION_CRES_SUSPENDED;
   }
   if ( (GNUNET_OS_PROCESS_EXITED != as->pst) ||
        (0 != as->exit_code) )
@@ -444,8 +432,8 @@ sms_process (struct ANASTASIS_AUTHORIZATION_State *as,
                                        TALER_EC_ANASTASIS_SMS_HELPER_COMMAND_FAILED,
                                        es);
     if (MHD_YES != mres)
-      return ANASTASIS_AUTHORIZATION_RES_FAILED_REPLY_FAILED;
-    return ANASTASIS_AUTHORIZATION_RES_FAILED;
+      return ANASTASIS_AUTHORIZATION_CRES_FAILED_REPLY_FAILED;
+    return ANASTASIS_AUTHORIZATION_CRES_FAILED;
   }
 
   /* Build HTTP response */
@@ -464,12 +452,9 @@ sms_process (struct ANASTASIS_AUTHORIZATION_State *as,
                                  "application/json"))
     {
       resp = TALER_MHD_MAKE_JSON_PACK (
-        GNUNET_JSON_pack_uint64 ("code",
-                                 TALER_EC_ANASTASIS_TRUTH_CHALLENGE_RESPONSE_REQUIRED),
-        GNUNET_JSON_pack_string ("hint",
-                                 TALER_ErrorCode_get_hint (
-                                   TALER_EC_ANASTASIS_TRUTH_CHALLENGE_RESPONSE_REQUIRED)),
-        GNUNET_JSON_pack_string ("detail",
+        GNUNET_JSON_pack_string ("challenge_type",
+                                 "TAN_SENT"),
+        GNUNET_JSON_pack_string ("tan_address_hint",
                                  end));
     }
     else
@@ -493,12 +478,12 @@ sms_process (struct ANASTASIS_AUTHORIZATION_State *as,
                                              "text/plain"));
     }
     mres = MHD_queue_response (connection,
-                               MHD_HTTP_FORBIDDEN,
+                               MHD_HTTP_OK,
                                resp);
     MHD_destroy_response (resp);
     if (MHD_YES != mres)
-      return ANASTASIS_AUTHORIZATION_RES_SUCCESS_REPLY_FAILED;
-    return ANASTASIS_AUTHORIZATION_RES_SUCCESS;
+      return ANASTASIS_AUTHORIZATION_CRES_SUCCESS_REPLY_FAILED;
+    return ANASTASIS_AUTHORIZATION_CRES_SUCCESS;
   }
 }
 
@@ -596,7 +581,7 @@ libanastasis_plugin_authorization_sms_init (void *cls)
   plugin->cls = ctx;
   plugin->validate = &sms_validate;
   plugin->start = &sms_start;
-  plugin->process = &sms_process;
+  plugin->challenge = &sms_challenge;
   plugin->cleanup = &sms_cleanup;
 
   if (GNUNET_OK !=
