@@ -8,6 +8,7 @@ BOLD="$(tput bold)"
 NORM="$(tput sgr0)"
 
 set -eu
+set -x
 
 # Exit, with status code "skip" (no 'real' failure)
 function exit_skip() {
@@ -151,6 +152,7 @@ echo -n "Launching taler services ..."
 taler-bank-manage-testing $CONF postgres:///$TALER_DB serve > taler-bank.log 2> taler-bank.err &
 taler-exchange-secmod-eddsa -c $CONF 2> taler-exchange-secmod-eddsa.log &
 taler-exchange-secmod-rsa -c $CONF 2> taler-exchange-secmod-rsa.log &
+taler-exchange-secmod-cs -c $CONF 2> taler-exchange-secmod-cs.log &
 taler-exchange-httpd -c $CONF 2> taler-exchange-httpd.log &
 taler-merchant-httpd -c $CONF -L INFO 2> taler-merchant-httpd.log &
 taler-exchange-wirewatch -c $CONF 2> taler-exchange-wirewatch.log &
@@ -211,7 +213,7 @@ taler-exchange-offline -c $CONF \
   sign \
   enable-account payto://x-taler-bank/localhost/Exchange \
   enable-auditor $AUDITOR_PUB $AUDITOR_URL "TESTKUDOS Auditor" \
-  wire-fee now x-taler-bank TESTKUDOS:0.01 TESTKUDOS:0.01 \
+  wire-fee now x-taler-bank TESTKUDOS:0.01 TESTKUDOS:0.01 TESTKUDOS:0.01 \
   upload &> taler-exchange-offline.log
 
 echo -n "."
@@ -309,7 +311,7 @@ FEES=`jq -r -e '.upload_fees[0].fee' < $UFILE`
 # 4x 4.99 for annual fees, plus 4x0.01 for truth uploads
 if test "$FEES" != "TESTKUDOS:20"
 then
-    jq -e . $TFILE
+    jq -e . $UFILE
     exit_fail "Expected upload fees to be 'TESTKUDOS:20', got '$FEES'"
 fi
 
@@ -326,13 +328,18 @@ then
     exit_fail "Expected new state to be 'TRUTHS_PAYING', got '$STATE'"
 fi
 
-TMETHOD=`jq -r -e '.policies[0].methods[0].truth.type' < $TFILE`
-if test $TMETHOD != "question"
-then
-    exit_fail "Expected method to be >='question', got $TMETHOD"
-fi
+# FIXME: this test is specific to how the
+# C reducer stores state (redundantly!), should converge eventually!
+
+#TMETHOD=`jq -r -e '.policies[0].methods[0].truth.type' < $TFILE`
+#if test $TMETHOD != "question"
+#then
+#    exit_fail "Expected method to be >='question', got $TMETHOD"
+#fi
+#
+#echo " OK"
+
 
-echo " OK"
 #Pay
 
 echo -en $COLOR$BOLD"Withdrawing amount to wallet ..."$NORM$NOCOLOR
