@@ -26,6 +26,36 @@ function cleanup()
     wait
 }
 
+function sync_providers() {
+  infile=$1
+  outfile=$2
+  echo "Synchronizing providers"
+  # Sync with providers (up to 3 providers aren't synced here)
+  for x in 1 2 3; do
+    echo "Synchronizing providers (round $x)"
+    anastasis-reducer sync_providers < $infile > $outfile 2> /dev/null || true
+    CODE=$(jq -r -e ".code // 0" < $outfile)
+    # ANASTASIS_REDUCER_PROVIDERS_ALREADY_SYNCED
+    # FIXME: Temporary workaround for C reducer. See #7227.
+    if test "$CODE" = "8420"; then
+      # restore previous non-error state
+      cat $infile > $outfile
+      break
+    fi
+    # ANASTASIS_REDUCER_ACTION_INVALID
+    if test "$CODE" = "8400"; then
+      # restore previous non-error state
+      cat $infile > $outfile
+      break
+    fi
+    if test "$CODE" != "0"; then
+      exit_fail "Expected no error or 8420/8400, got $CODE"
+    fi
+    cat $outfile > $infile
+  done
+  echo "Providers synced."
+}
+
 
 CONF_1="test_anastasis_reducer_1.conf"
 CONF_2="test_anastasis_reducer_2.conf"
@@ -149,6 +179,7 @@ echo -n "Launching taler services ..."
 taler-bank-manage-testing $CONF postgres:///$TALER_DB serve > taler-bank.log 2> taler-bank.err &
 taler-exchange-secmod-eddsa -c $CONF 2> taler-exchange-secmod-eddsa.log &
 taler-exchange-secmod-rsa -c $CONF 2> taler-exchange-secmod-rsa.log &
+taler-exchange-secmod-cs -c $CONF 2> taler-exchange-secmod-cs.log &
 taler-exchange-httpd -c $CONF 2> taler-exchange-httpd.log &
 taler-merchant-httpd -c $CONF -L INFO 2> taler-merchant-httpd.log &
 taler-exchange-wirewatch -c $CONF 2> taler-exchange-wirewatch.log &
@@ -209,7 +240,7 @@ taler-exchange-offline -c $CONF \
   sign \
   enable-account payto://x-taler-bank/localhost/Exchange \
   enable-auditor $AUDITOR_PUB $AUDITOR_URL "TESTKUDOS Auditor" \
-  wire-fee now x-taler-bank TESTKUDOS:0.01 TESTKUDOS:0.01 \
+  wire-fee now x-taler-bank TESTKUDOS:0.01 TESTKUDOS:0.01 TESTKUDOS:0.01 \
   upload &> taler-exchange-offline.log
 
 echo -n "."
@@ -273,12 +304,11 @@ echo -n "Running backup logic ...,"
 anastasis-reducer -b > $B1FILE
 echo -n "."
 anastasis-reducer -a \
-  '{"continent": "Testcontinent"}' \
+  '{"continent": "Demoworld"}' \
   select_continent < $B1FILE > $B2FILE
 echo -n "."
 anastasis-reducer -a \
-  '{"country_code": "xx",
-    "currencies":["TESTKUDOS"]}' \
+  '{"country_code": "xx"}' \
   select_country < $B2FILE > $B1FILE
 echo -n "."
 anastasis-reducer -a \
@@ -287,6 +317,9 @@ anastasis-reducer -a \
     "sq_number": "4",
     "birthdate": "2000-01-01"}}' \
   enter_user_attributes < $B1FILE > $B2FILE
+cat $B2FILE > $B1FILE
+echo -n ","
+sync_providers $B1FILE $B2FILE
 echo -n ","
 # "91GPWWR" encodes "Hans"
 anastasis-reducer -a \
@@ -406,11 +439,10 @@ echo " OK"
 echo -n "Running recovery basic logic ..."
 anastasis-reducer -r > $R1FILE
 anastasis-reducer -a \
-  '{"continent": "Testcontinent"}' \
+  '{"continent": "Demoworld"}' \
   select_continent < $R1FILE > $R2FILE
 anastasis-reducer -a \
-  '{"country_code": "xx",
-    "currencies":["TESTKUDOS"]}' \
+  '{"country_code": "xx"}' \
   select_country < $R2FILE > $R1FILE
 anastasis-reducer -a '{"identity_attributes": { "full_name": "Max Musterman", "sq_number": "4", "birthdate": "2000-01-01" }}' enter_user_attributes < $R1FILE > $R2FILE
 
@@ -422,9 +454,12 @@ then
 fi
 echo " OK"
 
-echo -n "Selecting default secret"
-mv $R2FILE $R1FILE
-anastasis-reducer next < $R1FILE > $R2FILE
+echo -n "Adding provider (to ensure it is loaded)"
+anastasis-reducer -a '{"provider_url" : "http://localhost:8086/" }' add_provider < $R2FILE > $R1FILE
+echo " OK"
+
+echo -n "Selecting secret to recover"
+anastasis-reducer -a '{"attribute_mask": 0, "providers" : [ { "version": 1, "url" : "http://localhost:8086/" } ] }' select_version < $R1FILE > $R2FILE
 
 STATE=`jq -r -e .recovery_state < $R2FILE`
 if test "$STATE" != "CHALLENGE_SELECTING"
@@ -433,6 +468,10 @@ then
 fi
 echo " OK"
 
+cat $R2FILE > $R1FILE
+
+sync_providers $R1FILE $R2FILE
+
 echo -n "Running challenge logic ..."
 
 UUID0=`jq -r -e .recovery_information.challenges[0].uuid < $R2FILE`
