@@ -1,6 +1,7 @@
 #!/bin/bash
 
 set -eu
+#set -x
 
 # Exit, with status code "skip" (no 'real' failure)
 function exit_skip() {
@@ -46,11 +47,40 @@ function prepare_sandbox_account() {
         --account-name=$4 \
         --ebics-user-id=$1 \
         --ebics-host-id=$EBICS_HOST \
-        --ebics-partner-id=$2 \
-        --currency=$CURRENCY
+        --ebics-partner-id=$2
   echo " OK"
 }
 
+function sync_providers() {
+  infile=$1
+  outfile=$2
+  echo "Synchronizing providers"
+  # Sync with providers (up to 3 providers aren't synced here)
+  for x in 1 2 3; do
+    echo "Synchronizing providers (round $x)"
+    anastasis-reducer sync_providers < $infile > $outfile 2> /dev/null || true
+    CODE=$(jq -r -e ".code // 0" < $outfile)
+    # ANASTASIS_REDUCER_PROVIDERS_ALREADY_SYNCED
+    # FIXME: Temporary workaround for C reducer. See #7227.
+    if test "$CODE" = "8420"; then
+      # restore previous non-error state
+      cat $infile > $outfile
+      break
+    fi
+    # ANASTASIS_REDUCER_ACTION_INVALID
+    if test "$CODE" = "8400"; then
+      # restore previous non-error state
+      cat $infile > $outfile
+      break
+    fi
+    if test "$CODE" != "0"; then
+      exit_fail "Expected no error or 8420/8400, got $CODE"
+    fi
+    cat $outfile > $infile
+  done
+  echo "Providers synced."
+}
+
 
 # Install cleanup handler (except for kill -9)
 trap cleanup EXIT
@@ -160,7 +190,7 @@ fi
 echo " OK"
 
 echo -n "Starting Sandbox ..."
-libeufin-sandbox serve &> sandbox.log &
+libeufin-sandbox serve --no-auth &> sandbox.log &
 sandbox_pid=$!
 if ! curl -s --retry 5 --retry-connrefused $SANDBOX_URL > /dev/null; then
   exit_skip "Could not launch Sandbox"
@@ -300,12 +330,11 @@ echo -n "Running backup logic ...,"
 anastasis-reducer -b > $B1FILE
 echo -n "."
 anastasis-reducer -a \
-  '{"continent": "Testcontinent"}' \
+  '{"continent": "Demoworld"}' \
   select_continent < $B1FILE > $B2FILE
 echo -n "."
 anastasis-reducer -a \
-  '{"country_code": "xx",
-    "currencies":["TESTKUDOS"]}' \
+  '{"country_code": "xx" }' \
   select_country < $B2FILE > $B1FILE 2>> test_reducer.err
 echo -n "."
 
@@ -316,6 +345,9 @@ anastasis-reducer -a \
     "birthdate": "2000-01-01"}}' \
   enter_user_attributes < $B1FILE > $B2FILE 2>> test_reducer.err
 echo -n ","
+cat $B2FILE > $B1FILE
+sync_providers $B1FILE $B2FILE
+echo -n ","
 BASEIBAN=`echo -n $IBAN_DEBIT | gnunet-base32`
 anastasis-reducer -a \
   "$(jq -n '{ authentication_method: {
@@ -327,6 +359,19 @@ anastasis-reducer -a \
   )" \
   add_authentication < $B2FILE > $B1FILE 2>> test_reducer.err
 echo -n "."
+
+# "91GPWWR" encodes "Hans"
+anastasis-reducer -a \
+  '{"authentication_method": {
+    "type": "question",
+    "instructions": "What is your name?",
+    "challenge": "91GPWWR"
+    } }' \
+  add_authentication < $B1FILE > $B2FILE 2>> test_reducer.err
+echo -n "."
+
+mv $B2FILE $B1FILE
+
 # Finished adding authentication methods
 anastasis-reducer \
     next < $B1FILE > $B2FILE 2>> test_reducer.err
@@ -356,7 +401,7 @@ echo " OK"
 echo -n "Running recovery basic logic ..."
 anastasis-reducer -r > $R1FILE
 anastasis-reducer -a \
-  '{"continent": "Testcontinent"}' \
+  '{"continent": "Demoworld"}' \
   select_continent < $R1FILE > $R2FILE
 anastasis-reducer -a \
   '{"country_code": "xx",
@@ -372,9 +417,13 @@ then
 fi
 echo " OK"
 
-echo -n "Selecting default secret"
-mv $R2FILE $R1FILE
-anastasis-reducer next < $R1FILE > $R2FILE 2>> test_reducer.err
+echo -n "Adding provider (to ensure it is loaded)"
+anastasis-reducer -a '{"provider_url" : "http://localhost:8086/" }' add_provider < $R2FILE > $R1FILE
+echo " OK"
+
+echo -n "Selecting secret to recover"
+anastasis-reducer  -a '{"attribute_mask": 0, "providers" : [ { "version": 1, "url" : "http://localhost:8086/" } ] }' \
+  select_version < $R1FILE > $R2FILE 2>> test_reducer.err
 
 STATE=`jq -r -e .recovery_state < $R2FILE`
 if test "$STATE" != "CHALLENGE_SELECTING"
@@ -383,9 +432,27 @@ then
 fi
 echo " OK"
 
+cat $R2FILE > $R1FILE
+sync_providers $R1FILE $R2FILE
+
 echo -n "Running challenge selection logic ..."
 
-NAME_UUID=`jq -r -e .recovery_information.challenges[0].uuid < $R2FILE`
+UUID0=`jq -r -e .recovery_information.challenges[0].uuid < $R2FILE`
+UUID1=`jq -r -e .recovery_information.challenges[1].uuid < $R2FILE`
+UUID0Q=`jq -r -e .recovery_information.challenges[0].instructions < $R2FILE`
+UUID1Q=`jq -r -e .recovery_information.challenges[1].instructions < $R2FILE`
+
+if test "$UUID1Q" = 'What is your name?'
+then
+    NAME_UUID=$UUID1
+    IBAN_UUID=$UUID0
+else
+    NAME_UUID=$UUID0
+    IBAN_UUID=$UUID1
+fi
+
+echo "OK"
+echo -n "Solving first challenge ..."
 anastasis-reducer -a \
   "$(jq -n '
     {
@@ -395,15 +462,30 @@ anastasis-reducer -a \
   )" \
   select_challenge < $R2FILE > $R1FILE 2>> test_reducer.err
 
+anastasis-reducer -a '{"answer": "Hans"}' \
+  solve_challenge < $R1FILE > $R2FILE
+
+echo "OK"
+echo -n "Solving IBAN challenge ..."
+
+anastasis-reducer -a \
+  "$(jq -n '
+    {
+        uuid: $UUID
+    }' \
+    --arg UUID "$IBAN_UUID"
+  )" \
+  select_challenge < $R2FILE > $R1FILE 2>> test_reducer.err
 echo "OK"
 
-METHOD=`jq -r -e .challenge_feedback.\"$NAME_UUID\".method < $R1FILE`
-if test "$METHOD" != "iban"
+
+METHOD=`jq -r -e .challenge_feedback.\"$IBAN_UUID\".state < $R1FILE`
+if test "$METHOD" != "iban-instructions"
 then
-    exit_fail "Expected method to be 'iban', got ${METHOD}"
+    exit_fail "Expected method to be 'iban-instructions', got ${METHOD}"
 fi
 
-ACC=`jq -r -e .challenge_feedback.\"$NAME_UUID\".details.credit_iban < $R1FILE`
+ACC=`jq -r -e .challenge_feedback.\"$IBAN_UUID\".target_iban < $R1FILE`
 if test "$ACC" != ${IBAN_CREDIT}
 then
     exit_fail "Expected account to be ${IBAN_CREDIT}, got ${ACC}"
@@ -413,10 +495,10 @@ anastasis-reducer \
   back < $R1FILE > $R2FILE 2>> test_reducer.err
 
 
-AMOUNT=`jq -r -e .challenge_feedback.\"$NAME_UUID\".details.challenge_amount < $R1FILE`
-SUBJECT=`jq -r -e .challenge_feedback.\"$NAME_UUID\".details.wire_transfer_subject < $R1FILE`
+AMOUNT=`jq -r -e .challenge_feedback.\"$IBAN_UUID\".challenge_amount < $R1FILE`
+SUBJECT=`jq -r -e .challenge_feedback.\"$IBAN_UUID\".wire_transfer_subject < $R1FILE`
 
-echo -n "Performing authorization wire transfer ..."
+echo -n "Performing authorization wire transfer ${SUBJECT} ..."
 wire_transfer_to_anastasis "${AMOUNT}" "${SUBJECT}"
 
 echo " OK"
@@ -427,7 +509,7 @@ echo " OK"
 
 # Now we should get the secret...
 echo -n "Polling for recovery ..."
-anastasis-reducer poll < $R2FILE > $R1FILE
+anastasis-reducer poll -L INFO < $R2FILE > $R1FILE
 echo " OK"
 
 echo -n "Checking recovered secret ..."
