@@ -3,14 +3,14 @@
   Copyright (C) 2020, 2021 Anastasis SARL
 
   Anastasis is free software; you can redistribute it and/or modify it under the
-  terms of the GNU Lesser General Public License as published by the Free Software
+  terms of the GNU General Public License as published by the Free Software
   Foundation; either version 3, or (at your option) any later version.
 
   Anastasis is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-  A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more details.
+  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 
-  You should have received a copy of the GNU Affero General Public License along with
+  You should have received a copy of the GNU General Public License along with
   Anastasis; see the file COPYING.GPL.  If not, see <http://www.gnu.org/licenses/>
 */
 /**
@@ -108,8 +108,6 @@ handle_config_finished (void *cls,
                                  &acfg.business_name),
         GNUNET_JSON_spec_string ("version",
                                  &acfg.version),
-        GNUNET_JSON_spec_string ("currency",
-                                 &acfg.currency),
         GNUNET_JSON_spec_json ("methods",
                                &methods),
         GNUNET_JSON_spec_uint32 ("storage_limit_in_megabytes",
@@ -120,8 +118,8 @@ handle_config_finished (void *cls,
                                     &acfg.truth_upload_fee),
         TALER_JSON_spec_amount_any ("liability_limit",
                                     &acfg.liability_limit),
-        GNUNET_JSON_spec_fixed_auto ("server_salt",
-                                     &acfg.salt),
+        GNUNET_JSON_spec_fixed_auto ("provider_salt",
+                                     &acfg.provider_salt),
         GNUNET_JSON_spec_end ()
       };
 
@@ -131,6 +129,9 @@ handle_config_finished (void *cls,
                              NULL, NULL))
       {
         GNUNET_break_op (0);
+        json_dumpf (json,
+                    stderr,
+                    JSON_INDENT (2));
         response_code = 0;
         break;
       }
@@ -179,19 +180,6 @@ handle_config_finished (void *cls,
           break;
         }
       }
-      if ( (GNUNET_OK !=
-            TALER_amount_cmp_currency (&acfg.liability_limit,
-                                       &acfg.annual_fee)) ||
-           (0 !=
-            strcasecmp (acfg.currency,
-                        acfg.annual_fee.currency)) )
-      {
-        GNUNET_break_op (0);
-        GNUNET_JSON_parse_free (spec);
-        response_code = 0;
-        break;
-      }
-
       if (! json_is_array (methods))
       {
         GNUNET_break_op (0);
@@ -228,6 +216,9 @@ handle_config_finished (void *cls,
           }
         }
         acfg.methods = mcfg;
+        GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                    "Good backend found at `%s'\n",
+                    co->url);
         co->cb (co->cb_cls,
                 MHD_HTTP_OK,
                 &acfg);
