@@ -3,14 +3,14 @@
   Copyright (C) 2020, 2021 Anastasis SARL
 
   Anastasis is free software; you can redistribute it and/or modify it under the
-  terms of the GNU Lesser General Public License as published by the Free Software
+  terms of the GNU General Public License as published by the Free Software
   Foundation; either version 3, or (at your option) any later version.
 
   Anastasis is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-  A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more details.
+  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 
-  You should have received a copy of the GNU Affero General Public License along with
+  You should have received a copy of the GNU General Public License along with
   Anastasis; see the file COPYING.GPL.  If not, see <http://www.gnu.org/licenses/>
 */
 /**
@@ -164,10 +164,12 @@ run (void *cls,
     TALER_TESTING_cmd_withdraw_amount ("withdraw-coin-1",
                                        "create-reserve-1",
                                        "EUR:5",
+                                       0, /* age */
                                        MHD_HTTP_OK),
     TALER_TESTING_cmd_withdraw_amount ("withdraw-coin-2",
                                        "create-reserve-1",
                                        "EUR:5",
+                                       0, /* age */
                                        MHD_HTTP_OK),
     /**
      * Check the reserve is depleted.
@@ -278,7 +280,7 @@ run (void *cls,
                                             0, /* challenge index */
                                             "SomeTruth1",
                                             0,  /* mode */
-                                            ANASTASIS_CHALLENGE_STATUS_SOLVED),
+                                            ANASTASIS_CHALLENGE_ANSWER_STATUS_SOLVED),
 #if 0
     ANASTASIS_TESTING_cmd_challenge_answer ("challenge-answer-2",
                                             NULL, /* payment ref */
@@ -286,13 +288,13 @@ run (void *cls,
                                             1, /* challenge index */
                                             "SomeTruth2",
                                             0, /* mode */
-                                            ANASTASIS_CHALLENGE_STATUS_SOLVED),
+                                            ANASTASIS_CHALLENGE_ANSWER_STATUS_SOLVED),
 #endif
     ANASTASIS_TESTING_cmd_challenge_start ("challenge-start-3-pay",
                                            NULL,  /* payment ref */
                                            "recover-secret-1",
                                            2,  /* challenge index */
-                                           ANASTASIS_CHALLENGE_STATUS_PAYMENT_REQUIRED),
+                                           ANASTASIS_CHALLENGE_START_STATUS_PAYMENT_REQUIRED),
     TALER_TESTING_cmd_merchant_claim_order ("fetch-challenge-pay-proposal",
                                             merchant_url,
                                             MHD_HTTP_OK,
@@ -310,14 +312,14 @@ run (void *cls,
                                            "challenge-start-3-pay",  /* payment ref */
                                            "recover-secret-1",
                                            2,  /* challenge index */
-                                           ANASTASIS_CHALLENGE_STATUS_INSTRUCTIONS),
+                                           ANASTASIS_CHALLENGE_START_STATUS_FILENAME_PROVIDED),
     ANASTASIS_TESTING_cmd_challenge_answer ("challenge-answer-3",
                                             "challenge-start-3-pay", /* payment ref */
                                             "recover-secret-1",
                                             2, /* challenge index */
                                             "challenge-start-3-paid", /* answer */
                                             1, /* mode */
-                                            ANASTASIS_CHALLENGE_STATUS_SOLVED),
+                                            ANASTASIS_CHALLENGE_ANSWER_STATUS_SOLVED),
     ANASTASIS_TESTING_cmd_recover_secret_finish ("recover-finish-1",
                                                  "recover-secret-1",
                                                  GNUNET_TIME_UNIT_SECONDS),
@@ -338,6 +340,7 @@ run (void *cls,
     TALER_TESTING_cmd_exec_offline_sign_fees ("offline-sign-fees",
                                               CONFIG_FILE,
                                               "EUR:0.01",
+                                              "EUR:0.01",
                                               "EUR:0.01"),
     TALER_TESTING_cmd_check_keys_pull_all_keys ("refetch /keys",
                                                 1),
